using System;
using System.Runtime.InteropServices.ComTypes;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Drawing;
using DirectShowLib;
using Datastead;

namespace DatasteadDirectShow
{
    public class MultipurposeEncoderGraph : MultipurposeEncoder
    {
        // Application-defined message to notify app of filtergraph events
        public const int WM_GRAPHNOTIFY = 0x8000 + 1;

        public IVideoWindow m_videoWindow = null;
        public IMediaControl m_mediaControl = null;
        public IMediaEventEx m_mediaEventEx = null;
        public IGraphBuilder m_graphBuilder = null;
        public IBaseFilter m_DatasteadMultipurposeEncoder = null;
        public IDatasteadMultipurposeDirectShowEncoder m_DatasteadMultipurposeEncoderConfig = null;
        public ICaptureGraphBuilder2 m_captureGraphBuilder = null;

        public DsROTEntry m_rot = null;

        public delegate void AddToLog(string LogString);
        public AddToLog m_AddToLog = null;
        public Control m_DisplayWindow = null;
        public PictureBox m_PictureFromMemoryBitmap = null;

        public MultipurposeEncoderGraph()
        {
        }

        public void GetDevicesNames(Guid FilterCategory, ComboBox ComboBox)
        {
            IMoniker[] moniker = new IMoniker[1];
            ICreateDevEnum devEnum = (ICreateDevEnum)(new CreateDevEnum());
            IEnumMoniker classEnum = null;
            int hr = devEnum.CreateClassEnumerator(FilterCategory, out classEnum, CDef.None);
            DsError.ThrowExceptionForHR(hr);
            Marshal.ReleaseComObject(devEnum);

            bool Done = false;
            while (!Done)
            {
                if (classEnum.Next(1, moniker, IntPtr.Zero) == 0)
                {
                    Guid IID_IPropertyBag = typeof(IPropertyBag).GUID;
                    object Properties;
                    moniker[0].BindToStorage(null, null, ref IID_IPropertyBag, out Properties);
                    object ObjDeviceName;
                    (Properties as IPropertyBag).Read("FriendlyName", out ObjDeviceName, null);
                    Properties = null;
                    ComboBox.Items.Add(ObjDeviceName);
                    Marshal.ReleaseComObject(moniker[0]);
                }
                else
                {
                    Done = true;
                }
                if (ComboBox.Items.Count > 0)
                {
                    ComboBox.SelectedIndex = 0;
                }
            }
            Marshal.ReleaseComObject(classEnum);
        }

        public IBaseFilter CreateCaptureDeviceInstanceFromName(Guid FilterCategory, string DeviceName)
        {
            IMoniker[] moniker = new IMoniker[1];
            ICreateDevEnum devEnum = (ICreateDevEnum)(new CreateDevEnum());
            IEnumMoniker classEnum = null;
            int hr = devEnum.CreateClassEnumerator(FilterCategory, out classEnum, CDef.None);
            DsError.ThrowExceptionForHR(hr);
            Marshal.ReleaseComObject(devEnum);

            object CaptureDevice = null;

            bool Done = false;
            while (!Done)
            {
                if (classEnum.Next(1, moniker, IntPtr.Zero) == 0)
                {
                    Guid IID_IPropertyBag = typeof(IPropertyBag).GUID;
                    object Properties;
                    moniker[0].BindToStorage(null, null, ref IID_IPropertyBag, out Properties);
                    object ObjDeviceName;
                    (Properties as IPropertyBag).Read("FriendlyName", out ObjDeviceName, null);
                    if ((string)ObjDeviceName == DeviceName)
                    {
                        Guid IID_IBaseFilter = typeof(IBaseFilter).GUID;
                        moniker[0].BindToObject(null, null, ref IID_IBaseFilter, out CaptureDevice);
                        Done = true;
                    }
                    Properties = null;
                    Marshal.ReleaseComObject(moniker[0]);
                }
                else
                {
                    Done = true;
                }
            }
            Marshal.ReleaseComObject(classEnum);
            return (IBaseFilter)CaptureDevice;
        }

        public void GetInterfaces(System.IntPtr WindowHandle)
        {
            int hr = 0;

            // An exception is thrown if cast fail
            m_graphBuilder = (IGraphBuilder)new FilterGraph();

            // Add our graph to the running object table, which will allow
            // the GraphEdit application to "spy" on our graph
            // for debugging purpose, comment it for the release
            m_rot = new DsROTEntry(m_graphBuilder);

            m_captureGraphBuilder = (ICaptureGraphBuilder2)new CaptureGraphBuilder2();
            m_mediaControl = (IMediaControl)m_graphBuilder;
            m_videoWindow = (IVideoWindow)m_graphBuilder;
            m_mediaEventEx = (IMediaEventEx)m_graphBuilder;

            hr = m_mediaEventEx.SetNotifyWindow(WindowHandle, WM_GRAPHNOTIFY, IntPtr.Zero);
            DsError.ThrowExceptionForHR(hr);
        }

        public void QueryDatasteadMultipurposeEncoderConfig()
        {
            m_DatasteadMultipurposeEncoderConfig = (IDatasteadMultipurposeDirectShowEncoder)m_DatasteadMultipurposeEncoder;
        }

        public void CloseInterfaces() // invoked also in the Form1.Designer.cs -> dispose
        {
            if (m_mediaControl != null)
                m_mediaControl.Stop();

            if (m_mediaEventEx != null)
                m_mediaEventEx.SetNotifyWindow(IntPtr.Zero, WM_GRAPHNOTIFY, IntPtr.Zero);

            // Relinquish ownership (IMPORTANT!) of the video window.
            // Failing to call put_Owner can lead to assert failures within
            // the video renderer, as it still assumes that it has a valid
            // parent window.
            if (m_videoWindow != null)
            {
                m_videoWindow.put_Visible(OABool.False);
                m_videoWindow.put_Owner(IntPtr.Zero);
            }

            // Remove filter graph from the running object table
            if (m_rot != null)
            {
                m_rot.Dispose();
                m_rot = null;
            }

            // Release DirectShow interfaces
            if (m_DatasteadMultipurposeEncoderConfig != null) { Marshal.ReleaseComObject(m_DatasteadMultipurposeEncoderConfig); m_DatasteadMultipurposeEncoderConfig = null; }
            if (m_DatasteadMultipurposeEncoder != null) { Marshal.ReleaseComObject(m_DatasteadMultipurposeEncoder); m_DatasteadMultipurposeEncoder = null; }
            if (m_mediaControl != null) { Marshal.ReleaseComObject(m_mediaControl); m_mediaControl = null; }
            if (m_mediaEventEx != null) { Marshal.ReleaseComObject(m_mediaEventEx); m_mediaEventEx = null; }
            if (m_videoWindow != null) { Marshal.ReleaseComObject(m_videoWindow); m_videoWindow = null; }
            if (m_graphBuilder != null) { Marshal.ReleaseComObject(m_graphBuilder); m_graphBuilder = null; }
            if (m_captureGraphBuilder != null) { Marshal.ReleaseComObject(m_captureGraphBuilder); m_captureGraphBuilder = null; }
        }

        public void HandleGraphEvent()
        {
            int hr = 0;
            EventCode evCode;
            IntPtr evParam1, evParam2;

            if (m_mediaEventEx == null)
                return;

            while (m_mediaEventEx.GetEvent(out evCode, out evParam1, out evParam2, 0) == 0)
            {
                if ((int)evCode == EC_MPE_TERMINATED_SUCCESS)
                {
                    if (m_AddToLog != null) m_AddToLog("FFmpeg process successfully completed.");
                }
                if ((int)evCode == EC_MPE_TERMINATED_ERROR)
                {
                    if (m_AddToLog != null) m_AddToLog("FFmpeg exited with error code " + evParam1.ToString());
                }
                else if (evCode == EventCode.ErrorAbort)
                {
                    if (m_AddToLog != null) m_AddToLog("FFmpeg error!");
                }

                // Free event parameters to prevent memory leaks associated with
                // event parameter data.  While this application is not interested
                // in the received events, applications should always process them.
                hr = m_mediaEventEx.FreeEventParams(evCode, evParam1, evParam2);
                DsError.ThrowExceptionForHR(hr);

                // Insert event processing code here, if desired
            }
        }

        public void SetupVideoWindow()
        {
            if (m_DisplayWindow == null) return;
            int hr = 0;

            // Set the video window to be a child of the main window
            hr = m_videoWindow.put_Owner(m_DisplayWindow.Handle);
            DsError.ThrowExceptionForHR(hr);

            hr = m_videoWindow.put_WindowStyle(WindowStyle.Child | WindowStyle.ClipChildren | WindowStyle.ClipSiblings);
            DsError.ThrowExceptionForHR(hr);

            // Use helper function to position video window in client rect 
            // of main application window
            ResizeVideoWindow();

            // Make the video window visible, now that it is properly positioned
            hr = m_videoWindow.put_Visible(OABool.True);
            DsError.ThrowExceptionForHR(hr);
        }

        public void ResizeVideoWindow()
        {
            if (m_DisplayWindow == null) return;
            if (m_videoWindow == null) return;
            {
                m_videoWindow.SetWindowPosition(0, 0, m_DisplayWindow.ClientSize.Width, m_DisplayWindow.ClientSize.Height);
            }
        }


    }
}
